#define BYTEREG(a)  (*(volatile unsigned char*)a)
#define PEDIR       BYTEREG(0xfffff420)
#define PEDATA      BYTEREG(0xfffff421)
#define PEUP        BYTEREG(0xfffff422)
#define PESEL       BYTEREG(0xfffff423)
#define PJDIR       BYTEREG(0xfffff438)
#define PJDATA      BYTEREG(0xfffff439)
#define PJSEL       BYTEREG(0xfffff43b)

#define LONGREG(a)  (*(volatile unsigned long*)a)
#define PORTE       LONGREG(0xfffff420)
#define PORTJ       LONGREG(0xfffff438)

/* interrupt mask register */
#define IMR  (*(unsigned long*)0xfffff304)

void getvals( short* val );

/* switch power pin on/off */
void adxl202_power(int on) { 
  if(on) PJDATA |=  0x80;
  else   PJDATA &= ~0x80;
}

/* switch self test pin on/off */
void adxl202_selftest(int on) {
  if(on) PJDATA |=  0x40;
  else   PJDATA &= ~0x40;
}

void adxl202_setup(void) {
  /* Port E */
  /* set upper two bits to gpio */
  PESEL |= 0x60; 

  /* disable pullups */
  PEUP  &= ~0x60; 

  /* set both pins to input */
  PEDIR &= ~0x60;

  /* Port J */
  /* set upper two bits to gpio */
  PJSEL |= 0xc0; 

  /* set both pins to output */
  PJDIR |=  0xc0;

  /* both output pins low (chip off) */
  PJDATA &= ~0xc0;

  /* now switch on chip */
  adxl202_power(1);
  adxl202_selftest(0);
}

void adxl202_exit(void) {
  /* power off accelerometer */
  adxl202_power(0);
}

#define SAMPLE 1
int adxl202_measure(int *x, int *y) {
  unsigned long irq_save;
  unsigned short i,j;
  int ax=0,ay=0;
  int t2[SAMPLE];
  short a[SAMPLE][6];

  /* disable all irqs */
  irq_save = IMR;
  IMR = 0x00ffffff;

  for(i=0;i<SAMPLE;i++)
    getvals(a[i]);

  /* reenable irqs */
  IMR = irq_save;

  /* counter went down to 0?? -> no chip found */
  if((a[0][0]==-1)||(a[0][1]==-1)) return 0;

  for(i=0;i<SAMPLE;i++) {
    for(j=0;j<6;j++)  
      a[i][j] = 2000-a[i][j];

    /* calculate T2 */
    t2[i] = a[i][2]/2 + a[i][3] + a[i][4] + a[i][5]/2;

    ax += ((unsigned long)a[i][2]<<8)/t2[i];
    ay += ((unsigned long)a[i][5]<<8)/t2[i];
  }
  
  *x=(ax/SAMPLE); 
  *y=(ay/SAMPLE);

  return 1;
}

int adxl202_zero_x,adxl202_zero_y;
int adxl202_zero(void) {
  return(adxl202_measure(&adxl202_zero_x,&adxl202_zero_y));
}

void adxl202_get(int *x, int *y) {
  adxl202_measure(x,y);
  *x -= adxl202_zero_x;
  *y -= adxl202_zero_y;
}

// This asm was taken from original 'adxl202_asm.S' file
// provided by Till Harbaum. All I did was modify it
// to work with CodeWarrior... Gavin.
void asm getvals( short* val )
{
    link.w    a6,#0
	movem.l   d0-d5/a1,-(a7)
	
    move.l    8(a6),a0
	move.l	  #0xfffff421, a1  // Port E data reg

	move	  #2000,d0  // init div counter
	move	  d0,d1
	move	  d1,d2
	move	  d2,d3
	move	  d3,d4
	move	  d4,d5
		
loop_0:	btst    #6,(a1)	// wait while x high
	dbeq	  d0,loop_0

loop_1:	btst    #6,(a1)	// wait while x low
	dbne	  d1,loop_1

	// this is time Ta
	
loop_2:	btst    #6,(a1)	// wait while x high
	dbeq	  d2,loop_2

	// this is time Tb

loop_3:	btst    #5,(a1)	// wait while y high
	dbeq	  d3,loop_3
		
loop_4:	btst    #5,(a1)	// wait while y low
	dbne	  d4,loop_4

	// this is time Tc
	
loop_5:	btst    #5,(a1)	// wait while y high
	dbeq	  d5,loop_5

	// this is time Td

	move.w    d0,(a0)+
	move.w    d1,(a0)+
	move.w    d2,(a0)+
	move.w    d3,(a0)+
	move.w    d4,(a0)+
	move.w    d5,(a0)+
						
	movem.l   (a7)+,d0-d5/a1
	unlk      a6
	rts
}